<!---
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is comprised of the Starter App directory

The Initial Developer of the Original Code is
PaperThin, Inc. Copyright(C) 2016.
All Rights Reserved.

By downloading, modifying, distributing, using and/or accessing any files
in this directory, you agree to the terms and conditions of the applicable
end user license agreement.
--->

<!---
/* *************************************************************** */
Author:
	PaperThin, Inc.
Name:
	$widget_configurator_props.cfm
Summary:
	Widget Option Configurator Props
History:
 	2016-05-20 - GAC - Created
	2017-02-09 - GAC - Fixed issues with Fields that have no options or one option, 
							 so the saved or default value is still passed via a hidden field
	2017-02-23 - GAC - Updated to write/read data to and from an INI config file
	2017-03-27 - GAC - Updated to use the ACF Workaround for Dynamic method names
	2017-03-30 - GAC - Updated to render MultiCheckbox fields from field options
	2017-04-17 - GAC - Updated the custom override notification text
	2017-04-25 - GAC - Update to allow multicheckbox to have one item
--->
<cfsetting enablecfoutputonly="Yes" showdebugoutput="No">

<!--- if this module loads resources, do it here.. --->
<cfscript>
   application.ADF.scripts.loadJQuery();
	application.ADF.scripts.loadJQueryUI();
	application.ADF.scripts.loadFontAwesome();
	//application.ADF.scripts.loadJQueryHighlightTextArea();
</cfscript>

<!--- ... then exit if all we're doing is detecting required resources --->
<cfif Request.RenderState.RenderMode EQ "getResources">
  <cfexit>
</cfif>

<cfscript>
	// Variable for the version of the field - Display in Props UI.
	fieldVersion = "1.2.2";

	// initialize some of the attributes variables
	typeid = attributes.typeid;
	prefix = attributes.prefix;
	formname = attributes.formname;
	currentValues = attributes.currentValues;
	fieldID = '';
	formID = attributes.formid;
	elementFormName = "";
	fieldPropsEnabled = false;
	widgetScriptEditorEnabled = false;
	widgetScriptDirExists = false;
	errorMsg = "";
	
	siteDir = request.site.dir;
	
	/* WidgetScript INI folder Defaults */
	widgetDefaultName = "";
	widgetDefaultDir = "/_cs_widgets";
	widgetConfigDir = "/config";
	
	/* WidgetScript INI file Defaults */
	scriptFileExt = "ini";
	scriptFileName = "";
	scriptFileNameCustom = "";
	scriptFilePath = "";
	fullScriptDir = "";
	fullScriptPath = "";
	fullScriptPathCustom = "";
	fullScriptName = "";
	fullScriptNameCustom = "";
	
//WriteDump(var=dlgtype,expand=false);
//WriteDump(var=attributes,expand=false);
//WriteDump(var=currentValues,expand=false);


/* DEV ONLY 
removeSavedDirPath = false;
if ( removeSavedDirPath ) 
{
	structDelete(currentValues,"widgetScriptDirPath");
	structDelete(currentValues,"widgetScriptDirStatus");
}*/


	// Is this a valid dlgType?
	switch (dlgtype)
	{
		case 'customelement': case 'metadata':
			fieldPropsEnabled = true;
			break;
		default:
			fieldPropsEnabled = false;
	}
	
	if ( fieldPropsEnabled )
	{
		// Is this a new field?
		if ( StructKeyExists(attributes,"fieldid") AND IsNumeric(attributes.fieldid) AND attributes.fieldid GT 0 )
			fieldid = attributes.fieldid;
	
		// If fieldid is a valid number or is greater than 0 we can enable the widget script editor 
		if ( IsNumeric(fieldid) AND fieldid GT 0 )
			widgetScriptEditorEnabled = true;
		
		// Get the Element Form Name for the specific form type
		elementFormName = getElementFormName(elementType=dlgtype,formID=attributes.formid);
//WriteDump(elementFormName);

		if ( LEN(TRIM(elementFormName)) EQ 0 )
		{
			errorMsg = "The Element Form name could not be resolved from provided Form ID!";
			fieldPropsEnabled = false;
		}
		else
		{
			// Build the Widget defualt name
			widgetDefaultName = LCASE(TRIM(REREPLACENOCASE(elementFormName,"[^A-Za-z0-9]","-","all")));
			
			// Build the Widget default script path
			widgetScriptDefaultDir = widgetDefaultDir & "/" & widgetDefaultName & widgetConfigDir & "/";
			fieldPropsEnabled = true;
		}
	}
	
	//Setup the default values
	defaultValues = StructNew();
	defaultValues.widgetScriptDirPath = widgetScriptDefaultDir;
	defaultValues.widgetScriptDirStatus = "";
	defaultValues.widgetScript = defaultScriptText();
	defaultValues.widgetScriptStatus = "";
	
	//defaultValues.defaultText = "";
	//defaultValues.useUdef = 0;
	//defaultValues.defaultValue = "";
	
//WriteDump("defaultValues.widgetScriptDirPath");
//WriteDump(defaultValues.widgetScriptDirPath);
	
	// Check if widgetScriptDirPath is defined and matches the default value
	if ( StructKeyExists(currentValues,"widgetScriptDirPath") )
	{
	
//WriteDump("currentValues.widgetScriptDirPath"); 
//WriteDump(currentValues.widgetScriptDirPath);

		// Clear the saved Dir STATUS value
		if ( StructKeyExists(currentValues,"widgetScriptDirStatus") )
			currentValues.widgetScriptDirStatus = "";
		
		if ( LEN(TRIM(currentValues.widgetScriptDirPath)) EQ 0 )
		{
			defaultValues.widgetScriptDirStatus = "new";
			widgetScriptEditorEnabled = false;
		}
		else if ( currentValues.widgetScriptDirPath EQ defaultValues.widgetScriptDirPath  ) 
			defaultValues.widgetScriptDirStatus = "default";
		else
			defaultValues.widgetScriptDirStatus = "custom";
	}
	else
	{
		defaultValues.widgetScriptDirStatus = "new";
		widgetScriptEditorEnabled = false;	
	}
	
//WriteDump("widgetScriptDirStatus"); 
//WriteDump(defaultValues.widgetScriptDirStatus);

   //This will override the default values with the current values.
   //In normal use this should not need to be modified.
   currentValueArray = StructKeyArray(currentValues);
   for(i=1;i<=ArrayLen(currentValueArray);i++){
      //If there is a default value set AND there is a current value set update the default value with the current value
      if ( StructKeyExists(defaultValues,currentValueArray[i]) ) {
         defaultValues[currentValueArray[i]] = currentValues[currentValueArray[i]];
      }
   }

//WriteDump(var=defaultValues,expand=false,label="default");
//WriteDump(var=currentValues,expand=false,label="current");



	//if ( !StructKeyExists(currentValues,"widgetScriptStatus") OR LEN(TRIM(defaultValues.widgetScriptStatus)) EQ 0 ) 
	//	defaultValues.widgetScriptStatus = "";
	//if ( !StructKeyExists(currentValues,"widgetScriptDirStatus") OR LEN(TRIM(defaultValues.widgetScriptDirStatus)) EQ 0 ) 
	//	defaultValues.widgetScriptDirStatus = "notsaved";
	
	// TEMP - Use to reset script text to the DEFAULT
	//defaultValues.widgetScript = defaultScriptText();
	
	
	
	if ( fieldPropsEnabled )
	{
		if ( widgetScriptEditorEnabled )
		{
			// Build the WidgetScript config file name form the Form Name
			scriptFileName = widgetDefaultName & "-" & attributes.fieldid;
			
			scriptFileNameCustom = scriptFileName & "_custom";
	
			// Build the INI file for Widget Configuration Form Fields
			scriptFilePath = 	defaultValues.widgetScriptDirPath;
	
			if ( Left(scriptFilePath,1) EQ "/" )
				siteDir = reReplace(request.site.dir,"/$","");
	
			if ( Right(scriptFilePath,1) NEQ "/" )
				scriptFilePath = scriptFilePath & "/";
		
			// Hard Code the "config" directory - NOT USED
			//if ( ListLast(scriptFilePath,"/") NEQ "config" ) 
			//	scriptFilePath = scriptFilePath & "config/";
	
			fullScriptDir = siteDir & scriptFilePath;
			fullScriptName = scriptFileName & "." & scriptFileExt;
			fullScriptPath = siteDir & scriptFilePath & fullScriptName;
			
			fullScriptNameCustom = scriptFileNameCustom & "." & scriptFileExt;
			fullScriptPathCustom = siteDir & scriptFilePath & fullScriptNameCustom;

//WriteDump(var=scriptFileName,expand=false);
//WriteDump(var=scriptFileNameCustom,expand=false);	
//WriteDump(var=fullScriptPath,expand=false);
//WriteDump(var=fullScriptPathCustom,expand=false);
			
			// Check if Directory Exists
			if ( DirectoryExists(fullScriptDir) ) 
				widgetScriptDirExists = true;

			// If the INI file does not exist, create it and the folders needed
			if ( !FileExists(fullScriptPath) )
			{	
				if ( StructKeyExists(currentValues,"widgetScript") )
					fileWriteData = fixSavedDataFileSectionBrackets(str=defaultValues.widgetScript);
				else
					fileWriteData = fixDefaultDataFileSectionBrackets(str=defaultValues.widgetScript);
				
				fileWriteStatus = application.ADF.utils.writeINIfile(filePath=fullScriptPath,dataString=fileWriteData,overwrite=false,charSet="utf-8");
//WriteDump(fileWriteStatus);
			}	
				// Used ADF for the FileWrite so it will create the needed directories
				//FileWrite(fullScriptPath,defaultValues.widgetScript,"utf-8");
	
				// Read In the CUSTOM file - NOT USED
				/*
					if ( FileExists(fullScriptPathCustom) ) 
						defaultValues.widgetScript = FileRead(fullScriptPathCustom,"utf-8");
					else
						defaultValues.widgetScript = FileRead(fullScriptPath,"utf-8");
				*/
	
				// defaultValues.widgetScript = formatScriptText(str=defaultValues.widgetScript);
			
			defaultValues.widgetScript = FileRead(fullScriptPath,"utf-8");
			defaultValues.widgetScript = fixDisplaySectionBrackets(str=defaultValues.widgetScript);
		}
		else
		{
			// This Field is new... so display the default WigetScript syntax 
			// - Unless currentValues contains some existing widgetScript info
			if ( !StructKeyExists(currentValues,"widgetScript") )
				defaultValues.widgetScript = defaultScriptText();
			
			//  - Format for display in the Properties script window
			// defaultValues.widgetScript = formatScriptText(str=defaultValues.widgetScript);
		}
	
		// Get the cfmlEngine type
		cfmlEngine = server.coldfusion.productname;
		if ( !FindNoCase(cfmlEngine,'ColdFusion Server') )
		{
			// For Lucee Inject Extra linebreaks before each open bracket (since the textarea removes them)
			//defaultValues.widgetScript = REReplace(defaultValues.widgetScript, "(\r\n|\n\r|\n|\r)(\[)","&##013;&##010;&##013;&##010;[", "all"); 		/* &##013;&##010; */
			//defaultValues.widgetScript = REReplace(defaultValues.widgetScript, "(\r\n|\n\r|\n|\r)(##\[)","&##013;&##010;&##013;&##010;##[", "all"); 	/* &##013;&##010; */
		}
	}
	
//WriteDump(var=attributes,expand=false);
//WriteDump(var=defaultValues.widgetScriptDirPath,expand=false);
//WriteDump(var=fullScriptPath,expand=false);
//WriteDump(var=fullScriptPathCustom,expand=false);
//WriteDump(var=defaultValues,expand=false,label="default2");
</cfscript>

<!--- // Only Display Props for compatible element form types --->
<cfif fieldPropsEnabled>
	<cfoutput>
		<script language="JavaScript" type="text/javascript">
			// register the fields with global props object, this uses the name of the field
			fieldProperties['#typeid#'].paramFields = '#prefix#widgetScriptDirPath,#prefix#widgetScript'; //,#prefix#widgetScriptStatus,#prefix#widgetScriptDirStatus

			//fieldProperties['#typeid#'].paramFields = '#prefix#defaultText,#prefix#useUdef,#prefix#currentDefault';
			//fieldProperties['#typeid#'].defaultValueField = '#prefix#defaultValue';

			// allows this field to have a common onSubmit Validator
			fieldProperties['#typeid#'].jsValidator = '#prefix#doValidate';

			//Validation function, this specific instance checks to verify they entered a valid value.
			function #prefix#doValidate() {
		
				if ( jQuery("###prefix#widgetScriptDirPath").val().length == 0 ) {
					alert("Please enter a valid path to a widget script configuration directory path.");
					return false;
				}
			
				if ( jQuery("###prefix#widgetScriptStatus").val() == 'notsaved' ) {
					alert("Please save your changes to the current WidgetScript.");
					return false;
				}
				
				/* if ( jQuery("###prefix#widgetScriptDirStatus").val() == 'notsaved' ) {
					alert("Please save the WidgetScript Directory Path.");
					return false;
				}*/
			
				return true;
			}

			jQuery( function(){
				// Set the current value to a Temp variable
				var #prefix#widgetScript_original_data = jQuery('###prefix#widgetScript').val();
				var #prefix#widgetScriptDirPath_original_data = jQuery('###prefix#widgetScriptDirPath').val();
				
				if ( jQuery("###prefix#widgetScriptDirStatus").val() == 'new' ) 
					pathStatus_#prefix#('new',0);
				
				// Setup the WidgetScript SAVE button
				jQuery("###prefix#widgetScript_saveBtn").button();
				jQuery("button###prefix#widgetScript_saveBtn").attr("disabled", true).addClass("ui-state-disabled");
			
				// Add the Binding for the Click to the WidgetScript SAVE button
				jQuery("###prefix#widgetScript_saveBtn").bind("click", saveFile_#prefix#);

				jQuery('###prefix#widgetScript').bind("change keyup paste", function(e) {
					 if ( jQuery(this).val() != #prefix#widgetScript_original_data) 
					 {
						  //jQuery("label###prefix#fileMsg").text("Changed from '" + #prefix#widgetScript_original_data + "' to '" + jQuery(this).val() + "'");
						  //jQuery("label###prefix#fileMsg").text("Changed");
						  // text changed
						  //#prefix#widgetScript_original_data = jQuery(this).val();

						  fileStatus_#prefix#('notsaved');
					 }
				});
				
				jQuery('###prefix#widgetScriptDirPath').bind("change keyup paste", function(e) {
					 if ( jQuery(this).val() != #prefix#widgetScriptDirPath_original_data ) 
					 	pathStatus_#prefix#('notsaved');
					 else
					 	pathStatus_#prefix#('saved');
				});
			});
			
			// Set the WidgetScript FILE change status
			function fileStatus_#prefix#(status) {
				if ( status == 'notsaved' )
				{
					jQuery("i.file-saved").hide();
					jQuery("i.file-notsaved").show();
					jQuery("label###prefix#widgetScript_fileStatus").text('Changes have NOT been saved!').addClass('action-required');
					jQuery("button###prefix#widgetScript_saveBtn").attr("disabled", false).removeClass("ui-state-disabled");
					
					jQuery("###prefix#widgetScriptStatus").val(status);
				}
				else
				{
					jQuery("i.file-saved").show();
					jQuery("i.file-notsaved").hide();
					jQuery("label###prefix#widgetScript_fileStatus").text('Changes has been saved!').removeClass('action-required');
					jQuery("button###prefix#widgetScript_saveBtn").attr("disabled", true).addClass("ui-state-disabled");
					
					#prefix#widgetScript_original_data = jQuery('###prefix#widgetScript').val();
					jQuery("###prefix#widgetScriptStatus").val(status);
				}
			}

			function saveFile_#prefix#(){
				var iniFilePath = "#fullScriptPath#";
				var iniFileData = jQuery("###prefix#widgetScript").val();

				
				iniFileData = iniFileData.replace(/(\r\n|\r|\n){2,}/g, '$1\n');

				// make call to check path
				jQuery.post("#application.ADF.ajaxProxy#",
					{
						bean: "utils_2_0",
						method: "writeINIfile",
						filePath: iniFilePath,
						dataString: iniFileData,
						overwrite: 1
					},
					function(results)
					{
						//console.log(results);

						// show the results
						if ( results )
						{
							fileStatus_#prefix#('saved');
						}
						else
						{
							fileStatus_#prefix#('notsaved');
						}
					});

				return true;
			}
			
			// Set the WidgetScript PATH change status
			function pathStatus_#prefix#(status,resizeDialog) {
				resize = (typeof resize !== 'undefined') ?  resize : 1;
				
				if ( status == 'notsaved' )
				{
					jQuery("i.path-saved").hide();
					jQuery("i.path-notsaved").show();
					jQuery("span###prefix#widgetDir_pathStatus").text('Directory path has changed!').addClass('action-required');
					//jQuery('textarea###prefix#widgetScript').prop('disabled',true);
					
					jQuery("div###prefix#widgetScript_editorCell").hide();
					jQuery("div###prefix#widgetScript_editorMsgCell").show();
					
					jQuery("###prefix#widgetScriptDirStatus").val(status);
				}
				else if ( status == 'new' )
				{
					jQuery("i.path-saved").hide();
					jQuery("i.path-notsaved").show();
					jQuery("span###prefix#widgetDir_pathStatus").text('Directory path is NEW!').addClass('action-required');
					//jQuery('textarea###prefix#widgetScript').prop('disabled',true);
					
					jQuery("div###prefix#widgetScript_editorCell").hide();
					jQuery("div###prefix#widgetScript_editorMsgCell").show();
					
					jQuery("###prefix#widgetScriptDirStatus").val(status);
				}
				else
				{
					jQuery("i.path-saved").show();
					jQuery("i.path-notsaved").hide();
					jQuery("span###prefix#widgetDir_pathStatus").text('Saved path has been restored!').removeClass('action-required');
					//jQuery('textarea###prefix#widgetScript').prop('disabled',false);
					
					jQuery("div###prefix#widgetScript_editorCell").show();
					jQuery("div###prefix#widgetScript_editorMsgCell").hide();
					
					jQuery("###prefix#widgetScriptDirStatus").val(status);
				}
				
				if ( resize )
				{
					ResizeWindow();
					HandleOnResize_Props();	
				}
			}
		</script>

		<style>
			.mt4 {
				margin-top: 4px;
			}
			.mt6 {
				margin-top:6px;
			}
			.mt8 {
				margin-top:6px;
			}
		
			textarea###prefix#widgetScript {
				overflow: auto; /* overflow is needed */
				resize: vertical;
			}
			
			div###prefix#widgetScript_editorControlsTop{
				margin-top: 1em;
				margin-bottom: 0.75em;
			}
			div###prefix#widgetScript_editorControlsBottom {
				margin-top: 1em;
			}
			div###prefix#widgetScript_editorNotificationsTop {
				margin-bottom: 1em;
			}
			div###prefix#widgetScript_overrideFileInfo {
				padding-left: 2.4em;
			}
			
			i.file-saved,
			i.path-saved {
				display: none;
				color: ##009933;
				vertical-align: middle;
			}
			i.file-notsaved,
			i.path-notsaved {
				display: none;
				color: ##cc0000;
				vertical-align: middle;
			}
			i.path-exists {
				color: ##009933;
				vertical-align: middle;
			}
			
			.action-required {
				color: ##cc0000;
				font-weight: bold;
			}
			
			div###prefix#widgetScript_editorMsgCell {
				display: none;
			}
		</style>

		<table>
			<tr>
				<td class="cs_dlgLabelSmall" valign="top">
					<label for="#prefix#widgetScriptDirPath">WidgetScript Directory:</label>
				</td>
				<td class="cs_dlgLabelSmall">
					<div>Enter the relative path to this Widget's config directory.</div>
					
					<div class="mt4">
						<input type="text" id="#prefix#widgetScriptDirPath" name="#prefix#widgetScriptDirPath" value="#defaultValues.widgetScriptDirPath#" size="60" maxlength="255">
						<i class="path-saved fa fa-check fa-2x" aria-hidden="true"></i>
						<i class="path-notsaved fa fa-exclamation-triangle fa-1x" aria-hidden="true"></i>
						<span id="#prefix#widgetDir_pathStatus"></span>
					</div>
					
					<div class="mt4">Example: /_cs_widgets/{widget-name}/config/</div>
					<div class="mt8">Note: The required folders will be created if they don't already exist.</div>
					<!--- <div id="dirExistsStatus" class="mt8">
						<cfif widgetScriptDirExists>
							<i class="path-exists fa fa-check fa-2x" aria-hidden="true"></i>
							<span id="#prefix#widgetDir_pathExists">Directory Exists</span>
						</cfif>
					</div> --->
				
					<input type="hidden" id="#prefix#widgetScriptDirStatus" name="#prefix#widgetScriptDirStatus" value="#defaultValues.widgetScriptDirStatus#">
				</td>
			</tr>
						
			<!--- // Add a Row for Spacing --->
			<tr>
				<td class="cs_dlgLabelSmall" colspan="2">
					&nbsp; 
<!--- <cfdump var="#defaultValues#" expand=false label="default3"> --->
				</td>
			</tr>
		
			<tr>
				<td class="cs_dlgLabelSmall" valign="top">
					<label for="#prefix#widgetScript">Default WidgetScript:</label>
				</td>
			
			<cfif widgetScriptEditorEnabled AND LEN(TRIM(defaultValues.widgetScriptDirPath)) AND DirectoryExists(fullScriptDir)>
			
				<td class="cs_dlgLabelSmall">
					<div id="#prefix#widgetScript_editorMsgCell" class="cs_dlgLabelSmallBold">
						Save the WidgetScript Directory relative path above and reopen this dialog<br/>
						to edit the WidgetScript syntax for this widget!
					</div>
					<div id="#prefix#widgetScript_editorCell">
				
						<!--- // File Editor Controls Top --->
						<div id="#prefix#widgetScript_editorControlsTop">
							<button type="button" name="#prefix#widgetScript_saveBtn" id="#prefix#widgetScript_saveBtn"><i class="icon-saveBtn fa fa-floppy-o fa-1x" aria-hidden="true"></i></button>
							<i class="file-saved fa fa-check fa-2x" aria-hidden="true"></i>
							<i class="file-notsaved fa fa-exclamation-triangle fa-1x" aria-hidden="true"></i>
							<label id="#prefix#widgetScript_fileStatus">No changes. File is current.</label>
						</div>
				
					<!--- // Render Notification if a CUSTOM OVERRIDE FILE IS FOUND --->
					<cfif FileExists(fullScriptPathCustom)>
						<div id="#prefix#widgetScript_editorNotificationsTop">
							<span class="fa-stack fa-md">
							  <i class="fa fa-circle fa-stack-2x action-required"></i>
							  <i class="fa fa-exclamation fa-stack-1x fa-inverse"></i>
							</span>
							<!--- <i class="fa fa-exclamation action-required" aria-hidden="true"></i> --->
							<label id="#prefix#widgetScript_overrideFileStatus" class="action-required">
								Custom override file has been detected. Changes made in the editor below will be saved but will NOT render.
							</label>
							<div id="#prefix#widgetScript_overrideFileInfo"><strong>Override File:</strong> #fullScriptNameCustom#</div>
							<!--- <span class="fa-stack fa-md">
							  <i class="fa fa-circle fa-stack-2x action-required"></i>
							  <i class="fa fa-exclamation fa-stack-1x fa-inverse"></i>
							</span> --->
							<!--- <i class="fa fa-exclamation action-required" aria-hidden="true"></i> --->
						</div>
					</cfif>
						
						<div>Enter the WidgetScript Syntax used to generate the Widget's Option Fields configuration file.</div>
						
						<div class="mt4">
							<textarea id="#prefix#widgetScript" name="#prefix#widgetScript" cols="90" rows="20" wrap="on">#defaultValues.widgetScript#</textarea>
						<div>
						
						<div class="mt4">WidgetScript Syntax Help:</div>
						<div> - Use semi-colons (;) to delimit options. eg. optionvalue1;optionvalue2;optionvalue3</div>
						<div> - Use pipes (|) to delimit value/label pairs. eg. optionvalue1|Option Label 1;optionvalue2|Option Label 2</div>
				
						<input type="hidden" id="#prefix#widgetScriptStatus" name="#prefix#widgetScriptStatus" value="#defaultValues.widgetScriptStatus#">
					</div>
				</td>
			<cfelse>
				<td class="cs_dlgLabelSmallBold">
					Save the WidgetScript Directory relative path above and reopen this dialog<br/>
					to add the WidgetScript syntax for this widget!
					<input type="hidden" id="#prefix#widgetScript" name="#prefix#widgetScript" value="#defaultValues.widgetScript#">
					<input type="hidden" id="#prefix#widgetScriptStatus" name="#prefix#widgetScriptStatus" value="#defaultValues.widgetScriptStatus#">
				</td>
			</cfif>
		
			</tr>

			<!--- <tr>
				<td class="cs_dlgLabelSmall">
					<label for="#prefix#defaultValue">Default Text:</label>
				</td>
				<td class="cs_dlgLabelSmall">
					<input type="text" id="#prefix#defaultText" name="#prefix#defaultText" value="#defaultValues.defaultText#">
				</td>
			</tr> --->
		
			<!--- <cfinclude template="/commonspot/metadata/form_control/input_control/default_value.cfm"> --->
			
			<!--- // Display "currentValues.widgetScript" data  
						- Used for verifying and upgrading existing CFT prior to WidgetScript Directory --->
			<cfif !widgetScriptEditorEnabled AND StructKeyExists(currentValues,"widgetScript")>
			<tr>
				<td class="cs_dlgLabelSmall" colspan="2">
					&nbsp; 
				</td>
			</tr>
			<tr>
				<td class="cs_dlgLabelSmall" valign="top">
					<label for="#prefix#widgetScript">Saved WidgetScript:</label>
				</td>
				<td class="cs_dlgLabelSmall">
					<textarea id="#prefix#widgetScriptCurrent" name="#prefix#widgetScriptCurrent" cols="90" rows="10" wrap="on" readonly="readonly" style="background-color:##ddd">#currentValues.widgetScript#</textarea> <!--- disabled="disabled"--->
				</td>
			</tr>
			</cfif>
			
			<tr>
				<td class="cs_dlgLabelSmall" colspan="2" style="font-size:7pt;">
					<hr />
					ADF Custom Field v#fieldVersion#
				</td>
			</tr>
		</table>
	</cfoutput>
<cfelse>
	<cfoutput>
		<table>
			<tr>
				<td class="cs_dlgLabelSmallBold" colspan="2">
					This Field is not compatible with this Form type!
					<cfif LEN(TRIM(errorMsg))><br/>#errorMsg#</cfif>
				</td>
			</tr>
		</table>
	</cfoutput>
</cfif>

<!--- 
	getElementFormName(elementType,formID)
	
	- Return the Form Name for the current form (customelement or metadata)
--->
<cffunction name="getElementFormName" access="public" returntype="string" output="true">
	<cfargument name="elementType" type="string" required="true">
	<cfargument name="formID" type="numeric" required="true">
	
	<cfscript>
		var retStr = "";
		var infoObj = '';
		var infoObjType = '';
		var argData = StructNew();
		var infoData = '';
		var infoMethod = '';
		var argName = '';
		var infoDataKey = '';
		var validFormType = false;
		
		switch (arguments.elementType)
		{
			case 'customelement':
				infoObjType = "CustomElement";
				infoMethod = "getInfo";
				argName = "elementID";
				infoDataKey = "name";
				validFormType = true;
				break;
			case 'metadata':
				infoObjType = 'MetadataForm';
				infoMethod = "getForms";
				argName = "ID";
				infoDataKey = "formName";
				validFormType = true;
				break;
			case 'simpleform': // this is here so we remember this dlgtype
				validFormType = false;
				break;
			default:
				validFormType = false;
		}
		
		if ( validFormType )
		{
			argData[argName] = arguments.formid;
//WriteDump(argData);

			infoObj = Server.CommonSpot.ObjectFactory.getObject(infoObjType);
//WriteDump(infoObj);
			
			// Get the Element Info based on element type 
			infoData = getElementInfo(objType=infoObjType,methodName=infoMethod,args=argData);
//WriteDump(infoData);
			
			// USE the ACF Workaround for Dynamic method names instead (  getElementInfo() )
			//infoObj.staticName = infoObj[infoMethod];
			//infoData = infoObj.staticName(argumentCollection=argData); 
			//infoData = infoObj[infoMethod](argumentCollection=argData);
//WriteDump(infoData);

			if ( StructKeyExists(infoData,infoDataKey) )
				retStr = infoData[infoDataKey];
		}
		
		return retStr;
	</cfscript>
</cffunction>


<!--- 
	getElementInfo(objType,methodName,args)
	
	- Gets the Custom Element or Metadata Form info/data 
	
	// Uses an ACF Workaround for Dynamic method names
--->
<cffunction name="getElementInfo" access="public" returntype="any">
    <cfargument name="objType" required="true">
	 <cfargument name="methodName" required="true">
	 <cfargument name="args" required="false" default="#structNew()#">
	 
	 <cfscript>
	 	var result = '';
	 	var methodResult = "";
	 	var infoObj = '';
	 	var allowedElementTypes = "CustomElement,MetadataForm";
		var allowedMethods = "getInfo,getForms";
		var methodAllowed = false;
		
		if ( ListFind(allowedElementTypes,arguments.objType) )
		{
			infoObj =  Server.CommonSpot.ObjectFactory.getObject(arguments.objType);
			methodAllowed = true;
		}
		else
			methodAllowed = false;
		
		if ( methodAllowed AND ListFind(allowedMethods,arguments.methodName) )
			methodAllowed = true;
		else
			methodAllowed = false;
    </cfscript>
    
    <cfif methodAllowed >
    	<cfinvoke component="#infoObj#" method="#arguments.methodName#" argumentCollection="#arguments.args#" returnvariable="result">
    </cfif>
    
	 <cfreturn result>
</cffunction>

<!---
	defaultScriptText()

	widgetScript Syntax:

	[Config]
	fields = field1,field2,field2
		## Required
		## Semi-Colon Delimited List of fields 
		## The Selection List Controls Render in this order

	[{fieldname}]
	group_{groupname} = option1;option2;option3
		## Optional
	group_all = option1|Option 1;option2|Option 2
		## Required
		## Semi-Colon Delimited List
	default = The default selected option
		## Optional
		## Omit or leave blank to set the first option to be the default
	description = Description text to be rendered under the selection list drop down control
		## Optional
	label = Field label text rendered to the left of the field control to be used instead of the {fieldname}
		## Optional

	Note:
	The {fieldname}'s in your widgetScript will need to be handled in the Render Handler to which the metadata form/field
	is bound. The {fieldname}'s will have all Non-AlphaNumeric character converted to underscores (_).
	eg. [Title Tag] will be Title_Tag and [Title-Color] will be Title_Color in the render handler's
	custom metadata form data.
--->
<cffunction name="defaultScriptText" access="public" returntype="string">

	<cfset var retVal = "">

<!--- 
	##
	## EXAMPLE WIDGET SCRIPT SYNTAX FILE
	## (Please remove this message and update before using!)
	##
--->

<cfsavecontent variable="retVal"><cfoutput>[Config]
fields=Theme

[Theme]
group_all=
default=
</cfoutput></cfsavecontent>

<!---  <cfsavecontent variable="retVal"><cfoutput>[Config]
fields=Theme;Columns X-Small;Columns Small;Columns Medium;Columns Large;Title Tag,Title Color,Title Align,Description Align

[Theme]
group_all=
default=

[Columns X-Small]
group_all=none;col-xs-12|1 column;col-xs-6|2 columns;col-xs-4|3 columns
description=The number of columns to render when the device size is Extra Small (Phones).

[Columns Small]
group_all=none;col-sm-12|1 column;col-sm-6|2 columns;col-sm-4|3 columns;col-sm-3|4 columns
description=The number of columns to render when the device size is Small (Tablets).

[Columns Medium]
group_all=none;col-md-12|1 column;col-md-6|2 columns;col-md-4|3 columns;col-md-3|4 columns
description=The number of columns to render when the device size is Medium (Desktop).

[Columns Large]
group_all=none;col-lg-12|1 column;col-lg-6|2 columns;col-lg-4|3 columns;col-lg-3|4 columns;col-lg-2|6 columns
description=The number of columns to render when the device size is Large (Large Desktop).

[Title Tag]
group_all=div;h1;h2;h3;h4;h5
default=H2
description=Select a Title Tag

[Title Align]
group_all=inherit;text-left|left;text-center|center;text-rgith|right
description=Select a Title Alignment

[Title Color]
group_admin-commonspot=inherit;red;yellow;orange;purple
group_all=inherent;red;yellow;orange
description=Select a Title Color

[Description Align]
group_all=inherit;text-left|left;text-center|center;text-rgith|right
description=Select a Description Alignment
</cfoutput></cfsavecontent>  --->

	<!--- <cfset retVal = REReplace(retVal, "(\r\n|\n\r|\n|\r)","&##013;&##010;", "all")> --->

	<cfreturn retVal>
</cffunction>

<!--- 
	formatScriptText(str)
	
	- Format for display in the properties textarea
--->
<!--- <cffunction name="formatScriptText" access="public" returntype="string">
	<cfargument name="str" type="string" required="false" default="">
	
	<cfreturn REReplace(arguments.str, "(\r\n|\n\r|\n|\r)","&##013;&##010;", "all")>
</cffunction> --->

<!--- 
	fixDisplaySectionBrackets(str)
	
	- Returns and newline before section headings
--->
<cffunction name="fixDisplaySectionBrackets" access="public" returntype="string">
	<cfargument name="str" type="string" required="false" default="">
	
	<cfscript>
		// Get the cfmlEngine type
		var cfmlEngine = server.coldfusion.productname;
		
		if ( !FindNoCase(cfmlEngine,'ColdFusion Server') )
		{
			arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(\[)","&##013;&##010;[", "all"); 			
			arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(##\[)","&##013;&##010;##[", "all"); 		
		}
		
			/* OLD 
			if ( !FindNoCase(cfmlEngine,'ColdFusion Server') )
			{
				// For Lucee Inject Extra linebreaks before each open bracket (since the textarea removes them)
				arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(\[)","&##013;&##010;&##013;&##010;[", "all"); 	
				arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(##\[)","&##013;&##010;&##013;&##010;##[", "all"); 	 
			}
			else
			{
				arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(\[)","&##013;&##010;[", "all"); 			
				arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(##\[)","&##013;&##010;##[", "all"); 		
			}*/
		
		return arguments.str;
	</cfscript>
</cffunction>

<!--- 
	fixDefaultDataFileSectionBrackets(str)
	
	- Updates string with return and newline before section headings for file saving
--->
<cffunction name="fixDefaultDataFileSectionBrackets" access="public" returntype="string">
	<cfargument name="str" type="string" required="false" default="">
	
	<cfscript>
		// Get the cfmlEngine type
		var cfmlEngine = server.coldfusion.productname;
		
		if ( !FindNoCase(cfmlEngine,'ColdFusion Server') )
		{
			arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(\[)","#CHR(13)##CHR(10)#[", "all"); 			
			arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(##\[)","#CHR(13)##CHR(10)###[", "all"); 		
		}
		
		return arguments.str;
	</cfscript>
</cffunction>

<!--- 
	fixSavedDataFileSectionBrackets(str)
	
	- For saved data updates string with return and newline before section headings for file saving
--->
<cffunction name="fixSavedDataFileSectionBrackets" access="public" returntype="string">
	<cfargument name="str" type="string" required="false" default="">
	
	<cfscript>
		// Get the cfmlEngine type
		var cfmlEngine = server.coldfusion.productname;
		
		if ( !FindNoCase(cfmlEngine,'ColdFusion Server') )
		{
			arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(\[)","#CHR(13)##CHR(10)##CHR(13)##CHR(10)#[", "all"); 			
			arguments.str = REReplace(arguments.str, "(\r\n|\n\r|\n|\r)(##\[)","#CHR(13)##CHR(10)##CHR(13)##CHR(10)###[", "all"); 		
		}
		
		return arguments.str;
	</cfscript>
</cffunction>